<?php

namespace Drupal\user_api_passwordless\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Flood\FloodInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\user_api_passwordless\Event\PasswordlessLoginAfterUserLoadEvent;
use Drupal\user_api_passwordless\Event\PasswordlessLoginBeforeUserLoadEvent;
use Drupal\user_api_passwordless\UserApiPasswordlessEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * Implement flood checks for passwordless login events.
 */
class PasswordlessLoginEventSubscriber implements EventSubscriberInterface {

  /**
   * User flood config.
   */
  protected ImmutableConfig $userFloodConfig;

  /**
   * Flood config.
   */
  protected ImmutableConfig $floodConfig;

  /**
   * Construct new EventSubscriber.
   */
  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected FloodInterface $flood,
    protected RequestStack $requestStack,
  ) {
    $this->userFloodConfig = $configFactory->get('user.flood');
    $this->floodConfig = $configFactory->get('user_api_passwordless.flood_login');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      UserApiPasswordlessEvents::PASSWORDLESS_LOGIN_BEFORE_USER_LOAD => [
        ['handleBeforeUserLoad'],
      ],
      UserApiPasswordlessEvents::PASSWORDLESS_LOGIN_AFTER_USER_LOAD => [
        ['handleAfterUserLoad'],
      ],
    ];
  }

  /**
   * Check if IP has been blocked before user load.
   */
  public function handleBeforeUserLoad(PasswordlessLoginBeforeUserLoadEvent $event) {
    // Check if attempts are exceeded.
    if (!$this->flood->isAllowed(
      'user_api_passwordless.login_request_ip',
      $this->floodConfig->get('ip_limit'),
      $this->floodConfig->get('ip_window')
    )) {
      $event->abort(
        'ip_blocked',
        new TranslatableMarkup('Too many login requests for your IP address.'),
        Response::HTTP_FORBIDDEN
      );

      // There is no return here on purpose,
      // becase a login request should always be registered!
    }

    // Always register a login request for ip.
    $this->flood->register(
      'user_api_passwordless.login_request_ip',
      $this->floodConfig->get('ip_window')
    );
  }

  /**
   * Check if user has been blocked after user load.
   */
  public function handleAfterUserLoad(PasswordlessLoginAfterUserLoadEvent $event) {
    $identifier = $this->getFloodUserIdentifier($event->user);

    // Check if attempts are exceeded.
    if (!$this->flood->isAllowed(
      'user_api_passwordless.login_request_user',
      $this->floodConfig->get('user_limit'),
      $this->floodConfig->get('user_window'),
      $identifier
    )) {
      $event->abort(
        'user_blocked',
        new TranslatableMarkup('Too many login requests for this account.'),
        Response::HTTP_FORBIDDEN
      );

      return;
    }

    // Register login request for user.
    $identifier = $this->getFloodUserIdentifier($event->user);
    $this->flood->register(
      'user_api_passwordless.login_request_user',
      $this->floodConfig->get('user_window'),
      $identifier
    );
  }

  /**
   * Get flood user identifier.
   */
  protected function getFloodUserIdentifier(AccountInterface $account) {
    $request = $this->requestStack->getCurrentRequest();

    if ($this->userFloodConfig->get('uid_only')) {
      return $account->id();
    }

    return $account->id() . '-' . $request->getClientIP();
  }

}
